/*-
 * Copyright (c) 2005 Fredrik Lindberg. <fli at shapeshifter dot se>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _DEBUG_H_
#define _DEBUG_H_

#ifdef DEBUG
# define DPRINTF(arg) do { \
	fprintf(stderr, ">DEBUG:%s:%d ", __FILE__, __LINE__); \
	printf arg; \
} while(0) 

#else
# define DPRINTF(arg) 
#endif

#define err_abort(text) do { \
	fprintf(stderr, "ABORT:%s:%d %s\n", __FILE__, __LINE__, text); \
	abort(); \
} while(0) 


#ifdef DEBUG
# define HEXDUMP(buf, _len) do { \
  { \
        size_t tmp; \
		char *__buf = (char *)buf; \
		printf(">DEBUG: "); \
        for (tmp = 0; tmp < _len; tmp++) \
                printf("%.2x ", *__buf++); \
	printf("\n"); \
  } \
} while(0)
#else
# define HEXDUMP(buf, len)
#endif

#endif /* _DEBUG_H_ */
